from zope.i18nmessageid import MessageFactory
workflownotificationMessageFactory = MessageFactory('raptus.workflownotification')
from Products.PlacelessTranslationService.utility import PTSTranslationDomain
workflownotificationdomain = PTSTranslationDomain('raptus.workflownotification')